@@ -1,4 +1,4 @@
-# Copyright (c) 2007-2008 Andrew Price
+# Copyright (c) 2007-2009 Andrew Price
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,7 @@
 
 import getpass
 import twitter
-import user
+import config
 import data
 from info import VERSION
 from optparse import OptionParser, make_option
@@ -42,14 +42,9 @@ class Commands:
 		self.parser = OptionParser(usage="%prog COMMAND [options]",
 				version=VERSION)
 
-		self.usr = user.User()
+		self.config = config.Config()
 		self.twitter = twitter.Twitter()
 		self.twitter.set_user_agent("Twyt")
-		if self.usr.getpassword() and self.usr.getusername():
-			self.twitter.set_auth(
-				self.usr.getusername(),
-				self.usr.getpassword())
-
 	
 	def register_command(self, command):
 		self.commands[str(command)] = command
@@ -83,19 +78,9 @@ class Commands:
 
 		options, args = self.__parse_command(command, argv[1:])
 
-		try:
-			if options.password is not None:
-				self.usr.setpassword(options.password)
-			elif options.user is not None:
-				self.usr.setusername(options.user)
-		except AttributeError:
-			# Just didn't ask for a user/password
-			pass
-
 		command.do_command(options, args, self)
 
-		self.usr.synctimes()
-
+		self.config.dirty_writeback()
 
 class Command:
 	def __init__(self):
@@ -156,18 +141,33 @@ class CommandWithAuth:
 
 		return pwd
 
-	def callwithauth(self, func, data, base):
+	def get_auth(self, options, base):
+		user = options.user or base.config.get_default_account()
+		pwd = options.password or base.config.get_password(user)
+
+		if not user:
+			# This exits
+			base.parser.error('No username specified.')
+		
+		if not pwd:
+			pwd = self.passwdprompt(user)
+
+		return (user, pwd)
+
+
+	def callwithauth(self, func, options, data, base, user, pwd):
+
+		if not pwd:
+			base.parser.error('No password specified.')
+		
+		base.twitter.set_auth(user, pwd)
+
 		answer = ""
 
-		while not answer:
-			try:
-				answer = func(*data)
-				break
-			except twitter.TwitterAuthException:
-				user = base.usr.getusername()
-				if not user:
-					base.parser.error('No username specified.')
-				base.twitter.set_auth(user, self.passwdprompt(user))
+		try:
+			answer = func(*data)
+		except twitter.TwitterAuthException:
+			base.parser.error('Bad username/password')
 
 		return answer
 
@@ -177,6 +177,10 @@ class TweetCommand(CommandWithAuth):
 		return "tweet"
 
 	def get_options(self):
+		self.options.append(make_option("-r", "--replyto",
+			metavar="ID", default=-1, type="int",
+			help="Explicitly sets the ID of the status message to"
+			"reply to."))
 		return self.options
 
 	def get_usage(self):
@@ -191,8 +195,9 @@ class TweetCommand(CommandWithAuth):
 
 		tweetmsg = " ".join(args)
 
-		answer = self.callwithauth(
-			base.twitter.status_update, [tweetmsg], base)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.status_update, options,
+				[tweetmsg, options.replyto], base, user, pwd)
 
 		result = data.Status()
 		result.load_json(answer)
@@ -222,8 +227,9 @@ class DirectCommand(CommandWithAuth):
 
 		touser = args[0]
 		msg = " ".join(args[1:])
-		answer = self.callwithauth(
-			base.twitter.direct_new, [touser, msg], base)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth( base.twitter.direct_new, options,
+				[touser, msg], base, user, pwd)
 
 		result = data.DirectMsg()
 		result.load_json(answer)
@@ -264,11 +270,12 @@ class DirectTLCommand(CommandWithAuth):
 			try:
 				since_id = int(options.since)
 			except ValueError:
-				since = arg
+				since = options.since
 	
-		answer = self.callwithauth(
-			base.twitter.direct_messages, [since, since_id,
-				options.page], base)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.direct_messages,
+				options, [since, since_id, options.page], base,
+				user, pwd)
 
 		results = data.DirectList(answer)
 		
@@ -308,8 +315,10 @@ class DirectSentCommand(CommandWithAuth):
 			except ValueError:
 				since = options.since
 
-		answer = self.callwithauth(base.twitter.direct_sent,
-				[since, since_id, options.page], base)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.direct_sent, options,
+				[since, since_id, options.page], base, user,
+				pwd)
 
 		results = data.DirectList(answer)
 		
@@ -342,8 +351,9 @@ class DirectDelCommand(CommandWithAuth):
 		if sid < 0:
 			base.parser.error("Invalid ID specified")
 
-		answer = self.callwithauth(base.twitter.direct_destroy,
-				[sid], base)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.direct_destroy, options,
+				[sid], base, user, pwd)
 		if answer:
 			result = data.DirectMsg()
 			result.load_json(answer)
@@ -358,11 +368,6 @@ class PublicTLCommand(Command):
 		return "publictl"
 	
 	def get_options(self):
-		self.options.append(
-			make_option("-i", "--since-id", metavar="ID",
-			type="int",
-			help="Returns only public statuses with an ID greater "
-			      "than (more recent than) the specified ID."))
 		return self.options
 
 	def get_usage(self):
@@ -373,25 +378,16 @@ class PublicTLCommand(Command):
 			"public timeline"
 
 	def do_command(self, options, args, base):
-		answer = base.twitter.status_public_timeline(
-					options.since_id)
 
+		answer = base.twitter.status_public_timeline()
 		results = data.StatusList(answer)
 
-		if options.since_id:
-			print "Since ID %s:" % options.since_id
-
 		for result in results:
 			print unicode(result)
 
-		if len(results) > 0:
-			base.usr.set_last_read('publictl', 'lastid',
-					str(results.get_last_id()))
-		else:
+		if len(results) == 0:
 			print "(Nothing new)"
 
-		return results
-
 
 class FriendsTLCommand(CommandWithAuth):
 	def __str__(self):
@@ -399,30 +395,41 @@ class FriendsTLCommand(CommandWithAuth):
 
 	def get_options(self):
 		self.options.append(make_option("-s", "--since",
-			metavar="DATE",
-			help="Narrows the returned results to just those "
-			      "statuses created after DATE"))
+			metavar="SINCE",
+			help="The date or ID of a message to list status "
+			"messages from."))
+		self.options.append(make_option("-P", "--page",
+			metavar="PAGE", type="int",
+			help="Lists the PAGEth page of status updates "))
 		return self.options
 
 	def get_usage(self):
-		return "%prog [options] [USERNAME]"
+		return "%prog [options]"
 
 	def get_help(self):
-		return "Returns 20 most recent statuses in your (or "\
-			"USERNAME's) friends timeline"
+		return "Returns 20 most recent statuses in your friends "\
+				"timeline"
 
 	def do_command(self, options, args, base):
-		ident = ""
-		if len(args) > 0:
-			ident = args[0]
 
-		since = base.usr.get_last_read('friendstl', ident)
+		user, pwd = self.get_auth(options, base)
 
-		if options.since is not None:
-			since = options.since
+		since_id = None
+		since = ""
+		if options.since:
+			try:
+				since_id = int(options.since)
+			except ValueError:
+				since = options.since
+
+		catchup = False
+		if not (since or since_id or options.page):
+			since_id = base.config.get_stamp(self, user)
+			catchup = True
 
 		answer = self.callwithauth(base.twitter.status_friends_timeline,
-				[ident, since], base)
+				options, [since, since_id, options.page], base,
+				user, pwd)
 
 		results = data.StatusList(answer)
 
@@ -431,9 +438,10 @@ class FriendsTLCommand(CommandWithAuth):
 		for result in results:
 			print unicode(result)
 
-		if len(results) > 0:
-			base.usr.set_last_read('friendstl', ident, results.get_last_time())
-		else:
+		if len(results) > 0 and catchup:
+			base.config.set_stamp('friendstl', user,
+					results.get_last_id())
+		elif catchup:
 			print "(Nothing new)"
 
 
@@ -444,7 +452,10 @@ class UserTLCommand(CommandWithAuth):
 	def get_options(self):
 		self.options.append(make_option("-s", "--since",
 			metavar="SINCE",
-			help="The date to list direct messages from."))
+			help="The date or ID to list statuses from."))
+		self.options.append(make_option("-P", "--page",
+			metavar="PAGE", type="int",
+			help="Lists the PAGEth page of status updates "))
 		self.options.append(make_option("-c", "--count",
 			metavar="COUNT", default=20, type="int",
 			help="The number of statuses to show, max 20"))
@@ -465,8 +476,18 @@ class UserTLCommand(CommandWithAuth):
 		if len(args) > 0:
 			ident = args[0]
 
+		since_id = None
+		since = ""
+		if options.since:
+			try:
+				since_id = int(options.since)
+			except ValueError:
+				since = options.since
+
+		user, pwd = self.get_auth(options, base)
 		answer = self.callwithauth(base.twitter.status_user_timeline,
-				[ident, count, options.since], base)
+				options, [ident, count, since, since_id,
+					options.page], base, user, pwd)
 		results = data.StatusList(answer)
 
 		for result in results:
@@ -493,7 +514,9 @@ class ShowCommand(CommandWithAuth):
 		else:
 			base.parser.error("No ID specified.")
 
-		answer = base.twitter.status_show(ident)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.status_show, options,
+				[ident], base, user, pwd)
 
 		if answer:
 			result = data.Status()
@@ -507,6 +530,9 @@ class RepliesCommand(CommandWithAuth):
 		return "replies"
 
 	def get_options(self):
+		self.options.append(make_option("-s", "--since",
+			metavar="SINCE",
+			help="The date or ID to list replies from."))
 		self.options.append(make_option("-P", "--page", metavar="PAGE",
 			type="int", default=1,
 			help="Shows the PAGEth page of replies."))
@@ -516,12 +542,22 @@ class RepliesCommand(CommandWithAuth):
 		return "%prog [options]"
 
 	def get_help(self):
-		return "Show the Nth page of 20 replies (messages with "\
+		return "Lists statuses which are replies to you (statuses with "\
 			"@yourusername in them)"
 
 	def do_command(self, options, args, base):
-		answer = self.callwithauth(base.twitter.status_replies,
-				[options.page], base)
+		user, pwd = self.get_auth(options, base)
+		since_id = None
+		since = ""
+		if options.since:
+			try:
+				since_id = int(options.since)
+			except ValueError:
+				since = options.since
+
+		answer = self.callwithauth(base.twitter.status_replies, options,
+				[options.page, since, since_id], base, user,
+				pwd)
 
 		if answer:
 			results = data.StatusList(answer)
@@ -554,8 +590,9 @@ class DeleteCommand(CommandWithAuth):
 		if sid < 0:
 			base.parser.error("Invalid ID specified")
 
-		answer = self.callwithauth(base.twitter.status_destroy,
-				[sid], base)
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.status_destroy, options,
+				[sid], base, user, pwd)
 		
 		if answer:
 			result = data.Status()
@@ -566,74 +603,142 @@ class DeleteCommand(CommandWithAuth):
 			print u'No messages deleted. Are you authorised?'
 
 
-class SavePassCommand(CommandWithAuth):
+class BlockCommand(CommandWithAuth):
 	def __str__(self):
-		return "savepass"
+		return "block"
 
 	def get_options(self):
 		return self.options
 
 	def get_usage(self):
-		return "%prog [options]"
+		return "%prog [options] ID"
 
 	def get_help(self):
-		return "Saves your username and password for subsequent use"
-
-	def do_command(self, _, __, base):
-		username = base.usr.getusername()
-		if username == "":
-			base.parser.error("Username not specified.")
-
-		pwd = self.passwdprompt(username)
-
-		base.usr.savepass(username, pwd)
-		base.twitter.set_auth(username, pwd)
-
-
-class ArchiveCommand(CommandWithAuth):
-	def __str__(self):
-		return "archive"
-
-	def get_options(self):
-		self.options.append(make_option("-P", "--page", metavar="PAGE",
-			default=0, type="int",
-			help="Retrieves page PAGE of your status messages"))
-		self.options.append(make_option("-s", "--since",metavar="DATE",
-			help="Narrows the returned results to just those "
-			      "statuses created after DATE"))
-		self.options.append(make_option("-i", "--since-id",
-			metavar="ID", type="int",
-			help="Returns only statuses with an ID greater "
-			      "than (more recent than) the specified ID."))
-		return self.options
-
-	def get_usage(self):
-		return "%prog [options]"
-
-	def get_help(self):
-		return "Shows 80 of your status messages ordered by date of "\
-			"posting"
+		return "Blocks a user specified by ID (numerical ID or"\
+				" screen name)"
 
 	def do_command(self, options, args, base):
-		page = ""
-		since = ""
-		since_id = ""
-
-		if options.page > 0:
-			page = options.page
-		if options.since:
-			since = options.since
-		if options.since_id:
-			since_id = options.since_id
-
 		ident = ""
-		if len(args) > 0:
-			ident = args[0]
+		if len(args) <= 0:
+			base.parser.error("ID not specified")
 
-		print [page, since, since_id]
-		answer = self.callwithauth(base.twitter.account_archive,
-				[page, since, since_id], base)
-		results = data.StatusList(answer)
+		ident = args[0]
+
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.block_create, options,
+				[ident], base, user, pwd)
+		if answer:
+			print "User ID %s blocked." % str(ident)
+
+
+class UnblockCommand(CommandWithAuth):
+	def __str__(self):
+		return "unblock"
+
+	def get_options(self):
+		return self.options
+
+	def get_usage(self):
+		return "%prog [options] ID"
+
+	def get_help(self):
+		return "Unblocks a user specified by ID (numerical ID or"\
+				" screen name)"
+
+	def do_command(self, options, args, base):
+		ident = ""
+		if len(args) <= 0:
+			base.parser.error("ID not specified")
+
+		ident = args[0]
+
+		user, pwd = self.get_auth(options, base)
+		answer = self.callwithauth(base.twitter.block_destroy, options,
+				[ident], base, user, pwd)
+		if answer:
+			print "User ID %s unblocked." % str(ident)
+
+class UserCommand(Command):
+	def __str__(self):
+		return "user"
+
+	def get_options(self):
+		self.options.append(
+			make_option("-u", "--user", metavar="USER",
+			default="",
+			help="Specify a Twitter username."))
+		self.options.append(
+			make_option("-p", "--pass", metavar="PASS",
+			default="", dest="password",
+			help="Specify a password."))
+		self.options.append(make_option("-l", "--list",
+			default=False, action="store_true",
+			help="Shows all stored usernames."))
+		self.options.append(make_option("-d", "--default",
+			default=False, action="store_true",
+			help="Set the given username as default."))
+		return self.options
+
+	def get_usage(self):
+		return "%prog [options] (set|unset)"
+
+	def get_help(self):
+		return "Get and set Twyt user options, e.g. remembered "\
+			"passwords and Twitter usernames"
+
+	def __print_accounts(self, accs=[], default=""):
+		for i,j in accs:
+			if j:
+				p = "password saved"
+			else:
+				p = "no password"
+			if i == default:
+				p += ", default"
+			print "%15s (%s)" % (i, p)
+
+	def __set_options(self, options, base):
+		if options.password and options.user:
+			base.config.set_password(options.user, options.password)
+			if options.default:
+				base.config.set_default(options.user)
+		elif options.user:
+			newacc = False
+			if not base.config.have_account(options.user):
+				base.config.add_account(options.user)
+				newacc = True
+			if options.default:
+				base.config.set_default_account(options.user)
+			if not options.default or newacc:
+				base.config.set_password(options.user,
+				  getpass.getpass(
+				    "Enter %s's Twitter password: " %\
+						    options.user))
+
+	def __unset_options(self, options, base):
+		if options.user:
+			base.config.remove_account(options.user)
+
+	def do_command(self, options, args, base):
+		d = base.config.get_default_account()
+		if options.list and options.user:
+			a = base.config.get_account(options.user)
+			if len(a) > 0:
+				self.__print_accounts(
+					[(options.user, bool(a['pass']))], d)
+			return
+		elif options.list:
+			self.__print_accounts(base.config.get_account_names(),d)
+			return
+
+		if len(args) == 1:
+			if args[0] == "set":
+				self.__set_options(options, base)
+			elif args[0] == "unset":
+				self.__unset_options(options, base)
+			else:
+				base.parser.error("Unknown directive: %s" %
+						args[0])
+			return
+		else:
+			base.parser.error("Insufficient options specified")
 
-		for result in results:
-			print unicode(result)
