@@ -1,4 +1,4 @@
-# Copyright (c) 2007-2008 Andrew Price
+# Copyright (c) 2007-2009 Andrew Price
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -84,7 +84,7 @@ class Status:
 			stamp = time.strftime("%a %b %d %H:%M:%S %Y",
 					time.localtime(utc))
 
-			string = "[%d] %s: %s (%s via %s)" % (self.id,
+			string = "[%d] %s: %s (%s via %s)" % (int(self.id),
 							self.user.name,
 							self.text,
 							stamp,
@@ -109,7 +109,13 @@ class Status:
 		self.text = p.save_end()
 		self.created_at = status[u'created_at']
 		self.id = status[u'id']
-		self.source = status[u'source']
+
+		p.save_bgn()
+		p.feed(status[u'source'])
+		self.source = p.save_end()
+		if self.source.endswith("[1]"):
+			self.source = self.source[:-3]
+
 		self.truncated = status[u'truncated']
 		self.user = User(status[u'user'])
 
