@@ -1,4 +1,4 @@
-# Copyright (c) 2007-2008 Andrew Price
+# Copyright (c) 2007-2009 Andrew Price
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -42,13 +42,14 @@ class TwitterAuthException(TwitterException):
 	pass # Nothing special about this exception
 
 class Twitter:
-
-	"""Provides a simple interface to the Twitter API"""
+	"""
+	Provides a simple interface to the Twitter API
+	"""
 
 	def __init__(self):
-
-		"""Initialise a Twitter API interface"""
-
+		"""
+		Initialise a Twitter API interface
+		"""
 		self.baseurl = u'http://twitter.com/'
 		#self.baseurl = u'http://localhost:8080/' # Test - 'nc -l -p 8080' ftw
 		self.auth = ""
@@ -56,27 +57,25 @@ class Twitter:
 
 
 	def set_user_agent(self, agent):
-
 		"""
 		Sets the user agent which is sent to Twitter in the
 		X-Twitter-Client header
 		"""
-
 		self.agent = str(agent)
 
 
 	def set_auth(self, user, pwd):
-
-		"""Set the user's Twitter credentials"""
-
+		"""
+		Set the user's Twitter credentials
+		"""
 		str64 = base64.encodestring('%s:%s' % (user,pwd))[:-1]
 		self.auth = u'Basic %s' % str64
 
 	def post(self, data, handler):
-
-		"""Sends a POST request to the given remote handler.
-		   The data argument must be a list of pairs of strings."""
-
+		"""
+		Sends a POST request to the given remote handler. The data
+		argument must be a list of pairs of strings.
+		"""
 		url = self.baseurl + handler
 		req = urllib2.Request(url, urllib.urlencode(data))
 
@@ -100,11 +99,10 @@ class Twitter:
 
 
 	def get(self, data, handler, doauth=False):
-
-		"""Sends a GET request to the given remote handler.
-		   The data argument must be a list of pairs of strings.
 		"""
-
+		Sends a GET request to the given remote handler.  The data
+		argument must be a list of pairs of strings.
+		"""
 		url = self.baseurl + handler + '?' + urllib.urlencode(data)
 		req = urllib2.Request(url)
 		req.add_header(u'X-Twitter-Client', self.agent)
@@ -119,19 +117,26 @@ class Twitter:
 		result = ""
 		try:
 			handle = urllib2.urlopen(req)
+			#print handle.code
 			result = handle.read()
 		except urllib2.HTTPError, e:
+			#print e.code
 			pass
 		except urllib2.URLError, e:
 			raise TwitterException("Connect failed: " + e.reason[1])
+		except httplib.BadStatusLine, e:
+			raise TwitterException(e.message)
 
 		return result
 
 
-	def status_update(self, msg=""):
-
-		"""Updates the user's status on twitter, otherwise known as tweeting"""
-		
+	def status_update(self, msg="", in_reply_to=None):
+		"""
+		Updates the user's status on twitter, otherwise known as
+		tweeting. in_reply_to (optional) explicitly tells Twitter which
+		status ID this message is a reply to.
+		"""
+		data = []
 		msg = msg.strip()
 		if len(msg) > 140:
 			raise TwitterException(
@@ -142,48 +147,53 @@ class Twitter:
 
 		data = [("status", msg), ("source", self.agent)]
 
+		if in_reply_to:
+			data.append(("in_reply_to_status_id", in_reply_to))
+
 		return self.post(data, u'statuses/update.json')
 
-	def status_public_timeline(self, since_id=""):
-		
-		"""Returns the 20 most recent statuses from non-protected users
-		   who have set a custom user icon. Does not require authentication.
+	def status_public_timeline(self):
+		"""
+		Returns the 20 most recent statuses from non-protected users
+		who have set a custom user icon. Does not require authentication.
 		"""
-
 		handler = u'statuses/public_timeline.json'
 
-		data = []
-		if since_id != "":
-			data = [("since_id", since_id)]
-
-		return self.get(data, handler, doauth=False)
+		return self.get([], handler, doauth=False)
 		
 
-	def status_friends_timeline(self, friend="", since=""):
-		
-		"""Returns the 20 most recent statuses posted in the last 24
-		   hours from the authenticating user and that user's friends.
+	def status_friends_timeline(self, since="", since_id="", page=1):
+		"""
+		Returns the 20 most recent statuses posted in the last 24
+		hours from the authenticating user and that user's friends.
 		"""
 
-		handler = u'statuses/friends_timeline'
-		if friend:
-			handler += u'/' + friend + u'.json'
-		else:
-			handler += u'.json'
-
+		try:
+			if page and int(page) < 1:
+				raise TwitterException("Page number is out of "\
+						"range: " + str(page))
+		except ValueError:
+			raise TwitterException("Invalid page number: " +\
+					str(page))
+		
+		handler = u'statuses/friends_timeline.json'
 		data = []
 		if since:
-			data = [("since", since)]
+			data.append(("since", since))
+		if since_id:
+			data.append(("since_id",since_id))
+		if page:
+			data.append(("page", page))
 
 		return self.get(data, handler, doauth=True)
 
-	def status_user_timeline(self, id="", count=20, since=""):
-
-		"""Returns the 20 (or count) most recent statuses posted in
-		   the last 24 hours from the authenticating user. It's also
-		   possible to request another user's timeline using id.
+	def status_user_timeline(self, id="", count=20, since="", since_id=None,
+			page=None):
+		"""
+		Returns the 20 (or count) most recent statuses posted by the
+		authenticating user. It's also possible to request another
+		user's timeline using id.
 		"""
-
 		handler = u'statuses/user_timeline'
 		if id != "":
 			handler += u'/' + id + u'.json'
@@ -192,7 +202,11 @@ class Twitter:
 
 		data = []
 		if since:
-			data = [("since", since)]
+			data.append(("since", since))
+		if since_id:
+			data.append(("since_id", since_id))
+		if page:
+			data.append(("page", page))
 
 		if 0 < count <= 20:
 			data.append(("count", count))
@@ -203,9 +217,9 @@ class Twitter:
 		return self.get(data, handler, doauth=True)
 
 	def status_show(self, id):
-		
-		"""Returns a single status, specified by the id parameter."""
-
+		"""
+		Returns a single status, specified by the id parameter.
+		"""
 		if not id:
 			raise TwitterException("No ID specified")
 
@@ -213,43 +227,43 @@ class Twitter:
 
 		return self.get([], handler, doauth=True)
 
-	def status_replies(self, page=1):
-
-		"""Returns the 20 most recent replies (status updates prefixed
-		   with @username posted by users who are friends with the user
-		   being replied to). The page argument gets the Nth 20 replies.
+	def status_replies(self, page=None, since="", since_id=None):
 		"""
-
-		if int(page) < 1:
-			raise TwitterException("Page number is out of range: "+\
-					str(page))
+		Returns the 20 most recent replies (status updates prefixed with
+		@username posted by users who are friends with the user being
+		replied to). The page argument gets the Nth 20 replies.
+		"""
+		data = []
+		if page:
+			data = [("page",page)]
+		if since:
+			data.append(("since", since))
+		if since_id:
+			data.append(("since_id", since_id))
 
 		handler = u'statuses/replies.json'
-		data = [("page",page)]
 
 		return self.get(data, handler, doauth=True)
 
 	def status_destroy(self, id):
-
-		"""Destroys the status specified by the required ID parameter.
-		   The authenticating user must be the author of the specified
-		   status.
 		"""
-
+		Destroys the status specified by the required ID parameter.
+		The authenticating user must be the author of the specified
+		status.
+		"""
 		if int(id) < 0:
 			raise TwitterException("ID out of range")
 
 		handler = u'statuses/destroy/%d.json' % id
 
-		return self.get([], handler, doauth=True)
+		return self.post([], handler)
 
 		
-	def direct_messages(self, since="", since_id="", page=1):
-
-		"""Returns a list of the 20 most recent direct messages sent to
-		   the authenticating user.
+	def direct_messages(self, since="", since_id="", page=None):
+		"""
+		Returns a list of the 20 most recent direct messages sent to the
+		authenticating user.
 		"""
-		
 		try:
 			if page and int(page) < 1:
 				raise TwitterException("Page number is out of "\
@@ -270,12 +284,11 @@ class Twitter:
 		return self.get(data, handler, doauth=True)
 
 
-	def direct_sent(self, since="", since_id="", page=1):
-
-		"""Returns a list of the 20 most recent direct messages sent by
-		   the authenticating user.
+	def direct_sent(self, since="", since_id="", page=None):
+		"""
+		Returns a list of the 20 most recent direct messages sent by the
+		authenticating user.
 		"""
-
 		try:
 			if page and int(page) < 1:
 				raise TwitterException("Page number is out of "\
@@ -296,12 +309,11 @@ class Twitter:
 		return self.get(data, handler, doauth=True)
 
 	def direct_new(self, user, text):
-
-		"""Sends a new direct message to the specified user from the
-		   authenticating user.  Requires both the user and text
-		   parameters.
 		"""
-
+		Sends a new direct message to the specified user from the
+		authenticating user.  Requires both the user and text
+		parameters.
+		"""
 		user = user.strip()
 		if not user:
 			raise TwitterException("User not specified")
@@ -320,27 +332,25 @@ class Twitter:
 
 
 	def direct_destroy(self, id):
-
-		"""Destroys the direct message specified in the required ID
-		   parameter.  The authenticating user must be the recipient of
-		   the specified direct message.
 		"""
-
+		Destroys the direct message specified in the required ID
+		parameter.  The authenticating user must be the recipient of the
+		specified direct message.
+		"""
 		if int(id) < 0:
 			raise TwitterException("ID out of range")
 
 		handler = u'direct_messages/destroy/%d.json' % id
 
-		return self.get([], handler, doauth=True)
+		return self.post([], handler)
 
-	def user_friends(self, id=None):
-
-		"""Returns up to 100 of the authenticating user's friends who
-		   have most recently updated, each with current status inline.
-		   It's also possible to request another user's recent friends
-		   list via the id parameter.
+	def user_friends(self, id=None, page=None, since=""):
+		"""
+		Returns up to 100 of the authenticating user's friends who have
+		most recently updated, each with current status inline.  It's
+		also possible to request another user's recent friends list via
+		the id parameter.
 		"""
-
 		if id is not None and id < 0:
 			raise TwitterException("User ID out of range")
 
@@ -349,64 +359,72 @@ class Twitter:
 		data = []
 		if id:
 			data.append(("id", id))
-
-		return self.get(data, handler, doauth=True)
-
-	def user_followers(self, lite=False):
-
-		"""Returns the authenticating user's followers, each with
-		   current status. If lite is True, current statuses are not
-		   returned.  (Note: At the time of writing, the lite parameter
-		   is broken in the Twitter API so this function will always
-		   return current statuses with the user data)
-		"""
-
-		data = []
-		if lite:
-			data.append(("lite", "true"))
-
-		handler = u'statuses/followers.json'
-
-		return self.get(data, handler, doauth=True)
-
-	def user_featured(self):
-
-		"""Returns a list of the users currently featured on the site
-		   with their current statuses inline.
-		"""
-
-		handler = u'statuses/featured.json'
-
-		return self.get([], handler, doauth=False)
-
-	def user_show(self, id):
-
-		"""Returns extended information of a given user, specified by
-		   ID or screen name with the id parameter.  This information
-		   includes design settings, so third party developers can
-		   theme their widgets according to a given user's preferences.
-		   The id parameter should be the ID or username of a user.
-		"""
-
-		handler = u'users/show/%s.json' % id
-
-		return self.get([], handler, doauth=False)
-
-	def account_archive(self, page=0, since="", since_id=""):
-
-		"""Returns 80 statuses per page for the authenticating user,
-		   ordered by descending date of posting.  Use this method to
-		   rapidly export your archive of statuses.
-		"""
-
-		handler = u'account/archive.json'
-
-		data = []
-		if page > 0:
-			data.append(("page", str(page)))
-		if since != "":
+		if page:
+			data.append(("page", page))
+		if since:
 			data.append(("since", since))
-		if since_id != "":
-			data.append(("since_id", since_id))
 
 		return self.get(data, handler, doauth=True)
+
+	def user_followers(self, id=None, page=None):
+		"""
+		Returns the authenticating user's followers, each with current
+		status. If id is given, returns the followers of the user
+		matching id (screen name or user id).
+		"""
+		data = []
+		if page:
+			data.append(("page", page))
+
+		handler = u'statuses/followers'
+
+		if id:
+			handler += u'/%s' % id
+
+		handler += u'.json'
+
+		return self.get(data, handler, doauth=True)
+
+	def user_show(self, id="", email=""):
+		"""
+		Returns extended information of a given user, specified by ID or
+		screen name with the id parameter.  This information includes
+		design settings, so third party developers can theme their
+		widgets according to a given user's preferences.  The id
+		parameter should be the ID or username of a user.
+		"""
+		data = []
+		handler = u'users/show'
+		if id:
+			handler += u'/%s.json' % id
+		elif email:
+			handler += u'.json'
+			data = [("email", email)]
+
+		return self.get(data, handler, doauth=True)
+
+	def block_create(self, id=None):
+		"""
+		Blocks the user specified in the ID parameter as the
+		authenticating user.  Returns the blocked user when successful.
+		"""
+		if id is None:
+			raise TwitterException("User ID not specified")
+
+		handler = u'blocks/create/%s.json' % str(id)
+
+		return self.post([], handler)
+
+
+	def block_destroy(self, id=None):
+		"""
+		Un-blocks the user specified in the ID parameter as the
+		authenticating user.  Returns the un-blocked user when
+		successful.
+		"""
+		if id is None:
+			raise TwitterException("User ID not specified")
+
+		handler = u'blocks/destroy/%s.json' % str(id)
+
+		return self.post([], handler)
