@@ -1,4 +1,4 @@
-# Copyright (c) 2007-2008 Andrew Price
+# Copyright (c) 2007-2009 Andrew Price
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -48,8 +48,9 @@ def main():
 	cmds.register_command(commands.ShowCommand())
 	cmds.register_command(commands.RepliesCommand())
 	cmds.register_command(commands.DeleteCommand())
-	cmds.register_command(commands.SavePassCommand())
-	cmds.register_command(commands.ArchiveCommand())
+	cmds.register_command(commands.BlockCommand())
+	cmds.register_command(commands.UnblockCommand())
+	cmds.register_command(commands.UserCommand())
 
 	try:
 		cmds.process_argv(sys.argv)
